use std::{
    collections::HashMap,
    path::{Path, PathBuf},
};

use anyhow::Context;

pub async fn process() -> anyhow::Result<()> {
    process_opts(ProcessOpts::default()).await
}

pub struct ProcessOpts {
    pub path: PathBuf,
}

impl Default for ProcessOpts {
    fn default() -> Self {
        Self {
            path: std::env::current_dir().expect("to be able to get current dir"),
        }
    }
}

pub async fn process_opts(opts: ProcessOpts) -> anyhow::Result<()> {
    let path = opts.path.canonicalize().context("failed to find folder")?;

    let cuddle_path = path.join("cuddle.yaml");
    let template = path.join("templates").join("clusters");

    tracing::debug!(
        "searching for templates in: {} with cuddle: {}",
        template.display(),
        cuddle_path.display()
    );

    let clusters = read_cuddle_section(&cuddle_path).await?;
    tracing::debug!("found clusters: {:?}", clusters);

    load_template_files(&template).await?;

    Ok(())
}

#[derive(serde::Deserialize, Default, Debug, Clone)]
struct CuddleClusters(HashMap<String, serde_yaml::Value>);

async fn read_cuddle_section(path: &Path) -> anyhow::Result<CuddleClusters> {
    let cuddle_file = tokio::fs::read(path).await?;

    let value: serde_yaml::Value = serde_yaml::from_slice(&cuddle_file)?;

    let mut cuddle_clusters = CuddleClusters::default();
    if let Some(clusters) = value.get("cuddle/clusters") {
        if let Some(mapping) = clusters.as_mapping() {
            for (key, value) in mapping.iter() {
                if let Some(key) = key.as_str() {
                    cuddle_clusters.0.insert(key.to_string(), value.clone());
                }
            }
        }
    }

    Ok(cuddle_clusters)
}

async fn load_template_files(path: &Path) -> anyhow::Result<()> {
    Ok(())
}
